#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
VisualWM 示例代码 - 文档水印 (Word, PDF, Excel, OFD)
"""

from visualwm import (
    WatermarkInfo,
    WatermarkStyle,
    RiskLevel,
    WordWatermark,
    PDFWatermark,
    ExcelWatermark,
    OFDWatermark,
)


def example_word_watermark():
    """Word文档水印示例"""
    print("=" * 60)
    print("示例1: Word文档水印 (.docx)")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="张三",
        employee_id="EMP001",
        department="研发部",
        auto_fill=True
    )
    
    # 创建Word水印处理器
    wm = WordWatermark()
    
    print("支持的水印类型:")
    print("  - header_footer: 页眉页脚水印")
    print("  - diagonal: 对角线文字水印")
    print("  - background: 背景水印")
    
    print("\n使用方法:")
    print("  wm = WordWatermark()")
    print("  wm.embed('input.docx', 'output.docx', info)")
    print("  wm.embed('input.docx', 'output.docx', info, watermark_type='diagonal')")
    
    # 演示创建新文档
    print("\n创建带水印的新Word文档:")
    print("  wm.embed_new('output.docx', '文档内容...', info)")
    
    print("✓ Word水印配置完成")


def example_pdf_watermark():
    """PDF文档水印示例"""
    print("\n" + "=" * 60)
    print("示例2: PDF文档水印 (.pdf)")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="李四",
        employee_id="EMP002",
        department="安全部",
        auto_fill=True
    )
    
    # 使用高风险样式
    style = WatermarkStyle.high_risk_preset()
    
    print(f"水印样式: 高风险")
    print(f"  前缀: {style.prefix}")
    print(f"  颜色: {style.hex_color}")
    
    # 创建PDF水印处理器
    wm = PDFWatermark()
    
    print("\n支持的水印类型:")
    print("  - tile: 平铺水印（默认）")
    print("  - center: 居中水印")
    print("  - diagonal: 对角线水印")
    print("  - header_footer: 页眉页脚水印")
    
    print("\n使用方法:")
    print("  wm = PDFWatermark()")
    print("  wm.embed('input.pdf', 'output.pdf', info, style)")
    print("  wm.embed('input.pdf', 'output.pdf', info, watermark_type='center')")
    
    # 加密PDF
    print("\n处理加密PDF:")
    print("  wm.embed('encrypted.pdf', 'output.pdf', info, password='密码')")
    
    # 获取PDF信息
    print("\n获取PDF信息:")
    print("  info = wm.get_pdf_info('document.pdf')")
    print("  # 返回: pages, width, height, encrypted, metadata")
    
    print("✓ PDF水印配置完成")


def example_excel_watermark():
    """Excel文档水印示例"""
    print("\n" + "=" * 60)
    print("示例3: Excel文档水印 (.xlsx)")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="王五",
        employee_id="EMP003",
        department="财务部",
        auto_fill=True
    )
    
    # 创建Excel水印处理器
    wm = ExcelWatermark()
    
    print("支持的水印类型:")
    print("  - header_footer: 页眉页脚水印（打印时显示）")
    print("  - first_row: 首行水印")
    print("  - comment: 批注水印")
    print("  - background: 背景列水印")
    
    print("\n使用方法:")
    print("  wm = ExcelWatermark()")
    print("  wm.embed('input.xlsx', 'output.xlsx', info)")
    print("  wm.embed('input.xlsx', 'output.xlsx', info, watermark_type='first_row')")
    
    # 指定工作表
    print("\n指定工作表:")
    print("  wm.embed('input.xlsx', 'output.xlsx', info, sheets=['Sheet1', 'Sheet2'])")
    
    # 创建新Excel
    print("\n创建带水印的新Excel文档:")
    print("  data = [['姓名', '工号'], ['张三', '001'], ['李四', '002']]")
    print("  wm.embed_new('output.xlsx', data, info)")
    
    # 获取工作簿信息
    print("\n获取工作簿信息:")
    print("  info = wm.get_workbook_info('document.xlsx')")
    print("  # 返回: sheet_names, sheet_count, sheets详情")
    
    print("✓ Excel水印配置完成")


def example_ofd_watermark():
    """OFD文档水印示例"""
    print("\n" + "=" * 60)
    print("示例4: OFD文档水印 (.ofd)")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="赵六",
        employee_id="EMP004",
        department="法务部",
        auto_fill=True
    )
    
    print("OFD (Open Fixed-layout Document):")
    print("  - 中国电子公文版式文档格式标准")
    print("  - 结构类似于ZIP压缩包")
    print("  - 包含XML描述和资源文件")
    
    # 创建OFD水印处理器
    wm = OFDWatermark()
    
    print("\n支持的水印类型:")
    print("  - annotation: 注释水印（默认）")
    print("  - layer: 图层水印")
    print("  - metadata: 元数据水印")
    
    print("\n使用方法:")
    print("  wm = OFDWatermark()")
    print("  wm.embed('input.ofd', 'output.ofd', info)")
    print("  wm.embed('input.ofd', 'output.ofd', info, watermark_type='metadata')")
    
    # 验证OFD文件
    print("\n验证OFD文件:")
    print("  is_valid = wm.is_valid_ofd('document.ofd')")
    
    # 获取OFD信息
    print("\n获取OFD信息:")
    print("  info = wm.get_ofd_info('document.ofd')")
    print("  # 返回: file_size, documents, pages")
    
    print("✓ OFD水印配置完成")


def example_batch_processing():
    """批量处理示例"""
    print("\n" + "=" * 60)
    print("示例5: 批量文档处理")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="批量处理",
        employee_id="BATCH001",
        auto_fill=True
    )
    
    print("批量处理Word文档:")
    print("  wm = WordWatermark()")
    print("  processed = wm.batch_embed('./input/', './output/', info, recursive=True)")
    
    print("\n批量处理PDF文档:")
    print("  wm = PDFWatermark()")
    print("  processed = wm.batch_embed('./input/', './output/', info)")
    
    print("\n批量处理Excel文档:")
    print("  wm = ExcelWatermark()")
    print("  processed = wm.batch_embed('./input/', './output/', info)")
    
    print("\n批量处理OFD文档:")
    print("  wm = OFDWatermark()")
    print("  processed = wm.batch_embed('./input/', './output/', info)")
    
    print("✓ 批量处理配置完成")


def example_extract_watermark():
    """提取水印示例"""
    print("\n" + "=" * 60)
    print("示例6: 提取文档水印")
    print("=" * 60)
    
    print("从Word文档提取:")
    print("  wm = WordWatermark()")
    print("  info = wm.extract('document.docx')")
    print("  if info:")
    print("      print(f'用户: {info.username}')")
    
    print("\n从PDF文档提取:")
    print("  wm = PDFWatermark()")
    print("  info = wm.extract('document.pdf')")
    
    print("\n从Excel文档提取:")
    print("  wm = ExcelWatermark()")
    print("  info = wm.extract('document.xlsx')")
    
    print("\n从OFD文档提取:")
    print("  wm = OFDWatermark()")
    print("  info = wm.extract('document.ofd')")
    
    print("✓ 水印提取配置完成")


def example_complete_workflow():
    """完整工作流示例"""
    print("\n" + "=" * 60)
    print("示例7: 完整文档水印工作流")
    print("=" * 60)
    
    # 创建水印信息
    info = WatermarkInfo(
        username="安全管理员",
        employee_id="SEC001",
        department="信息安全部",
        device_id="WORKSTATION-001",
        auto_fill=True,
        custom_data={
            "密级": "机密",
            "有效期": "2026-12-31"
        }
    )
    
    # 使用高风险样式
    style = WatermarkStyle.high_risk_preset()
    
    print("1. 创建水印信息:")
    print(f"   用户: {info.username}")
    print(f"   工号: {info.employee_id}")
    print(f"   校验码: {info.checksum[:8]}")
    
    print("\n2. 选择高风险样式:")
    print(f"   前缀: {style.prefix}")
    print(f"   颜色: {style.hex_color}")
    
    print("\n3. 初始化处理器:")
    word_wm = WordWatermark(default_style=style)
    pdf_wm = PDFWatermark(default_style=style)
    excel_wm = ExcelWatermark(default_style=style)
    ofd_wm = OFDWatermark(default_style=style)
    print("   ✓ Word处理器")
    print("   ✓ PDF处理器")
    print("   ✓ Excel处理器")
    print("   ✓ OFD处理器")
    
    print("\n4. 添加水印:")
    print("   word_wm.embed('report.docx', 'report_wm.docx', info)")
    print("   pdf_wm.embed('contract.pdf', 'contract_wm.pdf', info)")
    print("   excel_wm.embed('data.xlsx', 'data_wm.xlsx', info)")
    print("   ofd_wm.embed('official.ofd', 'official_wm.ofd', info)")
    
    print("\n5. 验证水印:")
    print("   extracted = word_wm.extract('report_wm.docx')")
    print("   if extracted.username == info.username:")
    print("       print('水印验证通过')")
    
    print("\n✓ 完整工作流演示完成")


def main():
    """主函数"""
    print("\n" + "=" * 60)
    print("VisualWM 文档水印示例")
    print("支持格式: Word(.docx), PDF(.pdf), Excel(.xlsx), OFD(.ofd)")
    print("=" * 60 + "\n")
    
    example_word_watermark()
    example_pdf_watermark()
    example_excel_watermark()
    example_ofd_watermark()
    example_batch_processing()
    example_extract_watermark()
    example_complete_workflow()
    
    print("\n" + "=" * 60)
    print("所有文档水印示例演示完成!")
    print("=" * 60)
    
    print("\n快速使用:")
    print("  from visualwm import WordWatermark, PDFWatermark, ExcelWatermark, OFDWatermark")
    print("  from visualwm import WatermarkInfo, WatermarkStyle")
    print()
    print("  info = WatermarkInfo(username='张三', employee_id='001', auto_fill=True)")
    print("  style = WatermarkStyle.high_risk_preset()  # 可选")
    print()
    print("  # Word")
    print("  WordWatermark().embed('in.docx', 'out.docx', info)")
    print("  # PDF")
    print("  PDFWatermark().embed('in.pdf', 'out.pdf', info)")
    print("  # Excel")
    print("  ExcelWatermark().embed('in.xlsx', 'out.xlsx', info)")
    print("  # OFD")
    print("  OFDWatermark().embed('in.ofd', 'out.ofd', info)")
    print()


if __name__ == "__main__":
    main()



"""
校验码生成器 - 用于防篡改机制
"""

import hashlib
import hmac
from typing import Optional


class ChecksumGenerator:
    """
    校验码生成器
    
    提供校验码的生成与验证功能，用于检测水印是否被篡改。
    """
    
    # 默认密钥（生产环境应使用环境变量或配置文件）
    DEFAULT_SECRET_KEY = "visualwm_secret_2024"
    
    @classmethod
    def generate(
        cls, 
        content: str, 
        secret_key: Optional[str] = None,
        algorithm: str = "sha256"
    ) -> str:
        """
        生成校验码
        
        Args:
            content: 需要校验的内容
            secret_key: 密钥（可选）
            algorithm: 哈希算法
            
        Returns:
            校验码字符串
        """
        key = (secret_key or cls.DEFAULT_SECRET_KEY).encode('utf-8')
        content_bytes = content.encode('utf-8')
        
        if algorithm == "sha256":
            h = hmac.new(key, content_bytes, hashlib.sha256)
        elif algorithm == "sha512":
            h = hmac.new(key, content_bytes, hashlib.sha512)
        elif algorithm == "md5":
            h = hmac.new(key, content_bytes, hashlib.md5)
        else:
            raise ValueError(f"不支持的算法: {algorithm}")
        
        return h.hexdigest()
    
    @classmethod
    def generate_short(
        cls, 
        content: str, 
        length: int = 8,
        secret_key: Optional[str] = None
    ) -> str:
        """
        生成短校验码
        
        Args:
            content: 需要校验的内容
            length: 校验码长度
            secret_key: 密钥（可选）
            
        Returns:
            短校验码字符串
        """
        full_checksum = cls.generate(content, secret_key)
        return full_checksum[:length]
    
    @classmethod
    def verify(
        cls, 
        content: str, 
        checksum: str,
        secret_key: Optional[str] = None,
        algorithm: str = "sha256"
    ) -> bool:
        """
        验证校验码
        
        Args:
            content: 需要校验的内容
            checksum: 待验证的校验码
            secret_key: 密钥（可选）
            algorithm: 哈希算法
            
        Returns:
            校验是否通过
        """
        expected = cls.generate(content, secret_key, algorithm)
        
        # 支持短校验码验证
        if len(checksum) < len(expected):
            expected = expected[:len(checksum)]
        
        return hmac.compare_digest(expected, checksum)
    
    @classmethod
    def generate_with_timestamp(
        cls, 
        content: str, 
        timestamp: str,
        secret_key: Optional[str] = None
    ) -> str:
        """
        生成带时间戳的校验码
        
        Args:
            content: 需要校验的内容
            timestamp: 时间戳
            secret_key: 密钥（可选）
            
        Returns:
            校验码字符串
        """
        combined = f"{content}|{timestamp}"
        return cls.generate(combined, secret_key)


class TamperDetector:
    """
    篡改检测器
    
    用于检测水印是否被篡改，并记录篡改行为。
    """
    
    def __init__(self):
        self.tamper_log = []
    
    def check(
        self, 
        original_info: str, 
        current_info: str,
        original_checksum: str,
        current_checksum: Optional[str] = None
    ) -> dict:
        """
        检测篡改
        
        Args:
            original_info: 原始信息
            current_info: 当前信息
            original_checksum: 原始校验码
            current_checksum: 当前校验码（可选）
            
        Returns:
            检测结果字典
        """
        result = {
            "is_tampered": False,
            "content_modified": False,
            "checksum_valid": True,
            "details": []
        }
        
        # 检查内容是否被修改
        if original_info != current_info:
            result["content_modified"] = True
            result["is_tampered"] = True
            result["details"].append("内容已被修改")
        
        # 验证校验码
        if not ChecksumGenerator.verify(current_info, original_checksum):
            result["checksum_valid"] = False
            result["is_tampered"] = True
            result["details"].append("校验码验证失败")
        
        # 记录篡改行为
        if result["is_tampered"]:
            self._log_tamper(original_info, current_info, result)
        
        return result
    
    def _log_tamper(self, original: str, current: str, result: dict):
        """记录篡改行为"""
        from datetime import datetime
        
        log_entry = {
            "timestamp": datetime.now().isoformat(),
            "original_content": original[:100],  # 只记录前100字符
            "current_content": current[:100],
            "detection_result": result
        }
        self.tamper_log.append(log_entry)
    
    def get_tamper_log(self) -> list:
        """获取篡改日志"""
        return self.tamper_log.copy()
    
    def clear_log(self):
        """清空篡改日志"""
        self.tamper_log.clear()



"""
Excel文档水印处理器 - 支持.xlsx格式添加明水印
"""

import os
from pathlib import Path
from typing import Optional, Union, List
from openpyxl import Workbook, load_workbook
from openpyxl.drawing.image import Image as XLImage
from openpyxl.styles import Font, Alignment, PatternFill
from openpyxl.comments import Comment
from openpyxl.worksheet.header_footer import HeaderFooterItem

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel


class ExcelWatermark:
    """
    Excel文档水印处理器
    
    支持在Excel文档(.xlsx)中添加明水印，包括：
    - 页眉/页脚水印
    - 单元格背景水印
    - 批注水印
    - 首行/首列水印
    
    Features:
        - 支持.xlsx格式
        - 多工作表处理
        - 自定义样式
        - 批量处理
    """
    
    SUPPORTED_EXTENSIONS = {'.xlsx', '.xlsm'}
    
    def __init__(self, default_style: Optional[WatermarkStyle] = None):
        """
        初始化Excel水印处理器
        
        Args:
            default_style: 默认水印样式
        """
        self.default_style = default_style or WatermarkStyle.default()
    
    def embed(
        self,
        input_path: Union[str, Path],
        output_path: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        watermark_type: str = "header_footer",
        sheets: Optional[List[str]] = None
    ) -> str:
        """
        在Excel文档中嵌入水印
        
        Args:
            input_path: 输入文档路径
            output_path: 输出文档路径
            info: 水印信息
            style: 水印样式
            watermark_type: 水印类型 ("header_footer", "first_row", "comment", "background")
            sheets: 要处理的工作表名称列表，None表示所有工作表
            
        Returns:
            输出文件路径
        """
        input_path = Path(input_path)
        output_path = Path(output_path)
        style = style or self.default_style
        
        if not input_path.exists():
            raise FileNotFoundError(f"输入文件不存在: {input_path}")
        
        # 打开工作簿
        wb = load_workbook(input_path)
        
        # 确定要处理的工作表
        target_sheets = sheets if sheets else wb.sheetnames
        
        # 为每个工作表添加水印
        for sheet_name in target_sheets:
            if sheet_name in wb.sheetnames:
                ws = wb[sheet_name]
                
                if watermark_type == "header_footer":
                    self._add_header_footer_watermark(ws, info, style)
                elif watermark_type == "first_row":
                    self._add_first_row_watermark(ws, info, style)
                elif watermark_type == "comment":
                    self._add_comment_watermark(ws, info, style)
                elif watermark_type == "background":
                    self._add_background_watermark(ws, info, style)
                else:
                    raise ValueError(f"不支持的水印类型: {watermark_type}")
        
        # 保存
        output_path.parent.mkdir(parents=True, exist_ok=True)
        wb.save(output_path)
        
        return str(output_path)
    
    def embed_new(
        self,
        output_path: Union[str, Path],
        data: List[List[str]],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        sheet_name: str = "Sheet1"
    ) -> str:
        """
        创建带水印的新Excel文档
        
        Args:
            output_path: 输出文档路径
            data: 表格数据（二维列表）
            info: 水印信息
            style: 水印样式
            sheet_name: 工作表名称
            
        Returns:
            输出文件路径
        """
        output_path = Path(output_path)
        style = style or self.default_style
        
        # 创建新工作簿
        wb = Workbook()
        ws = wb.active
        ws.title = sheet_name
        
        # 添加数据
        for row_idx, row_data in enumerate(data, start=1):
            for col_idx, cell_value in enumerate(row_data, start=1):
                ws.cell(row=row_idx, column=col_idx, value=cell_value)
        
        # 添加水印
        self._add_header_footer_watermark(ws, info, style)
        self._add_first_row_watermark(ws, info, style, insert_row=True)
        
        # 保存
        output_path.parent.mkdir(parents=True, exist_ok=True)
        wb.save(output_path)
        
        return str(output_path)
    
    def _add_header_footer_watermark(
        self,
        ws,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加页眉页脚水印"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 设置页眉
        ws.oddHeader.center.text = watermark_text
        ws.oddHeader.center.size = int(style.font_size * 0.6)
        ws.oddHeader.center.font = style.font_name
        
        # 设置页脚
        ws.oddFooter.center.text = watermark_text
        ws.oddFooter.center.size = int(style.font_size * 0.6)
        ws.oddFooter.center.font = style.font_name
        
        # 偶数页
        ws.evenHeader.center.text = watermark_text
        ws.evenFooter.center.text = watermark_text
    
    def _add_first_row_watermark(
        self,
        ws,
        info: WatermarkInfo,
        style: WatermarkStyle,
        insert_row: bool = False
    ):
        """添加首行水印"""
        watermark_text = self._format_watermark_text(info, style)
        
        if insert_row:
            ws.insert_rows(1)
        
        # 合并首行单元格
        max_col = ws.max_column if ws.max_column > 1 else 5
        
        # 写入水印
        cell = ws.cell(row=1, column=1, value=watermark_text)
        
        # 设置样式
        cell.font = Font(
            name=style.font_name,
            size=int(style.font_size * 0.7),
            bold=style.bold,
            italic=style.italic,
            color=f"{style.color[0]:02x}{style.color[1]:02x}{style.color[2]:02x}"
        )
        cell.alignment = Alignment(horizontal='center', vertical='center')
        
        # 合并单元格
        if max_col > 1:
            ws.merge_cells(start_row=1, start_column=1, end_row=1, end_column=max_col)
        
        # 设置行高
        ws.row_dimensions[1].height = 25
    
    def _add_comment_watermark(
        self,
        ws,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加批注水印（在A1单元格）"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 创建批注
        comment = Comment(
            f"【文档水印】\n{watermark_text}",
            "VisualWM"
        )
        comment.width = 300
        comment.height = 100
        
        # 添加到A1
        ws['A1'].comment = comment
    
    def _add_background_watermark(
        self,
        ws,
        info: WatermarkInfo,
        style: WatermarkStyle
    ):
        """添加背景水印（通过浅色填充实现提示效果）"""
        watermark_text = self._format_watermark_text(info, style)
        
        # 在工作表的显眼位置添加水印信息
        # 使用最后一列之后的位置
        max_col = ws.max_column + 2 if ws.max_column else 3
        max_row = ws.max_row if ws.max_row else 1
        
        # 添加水印列
        for row in range(1, max_row + 1, 10):  # 每10行添加一次
            cell = ws.cell(row=row, column=max_col, value=f"| {watermark_text}")
            cell.font = Font(
                size=8,
                color="CCCCCC"
            )
        
        # 设置列宽
        ws.column_dimensions[ws.cell(row=1, column=max_col).column_letter].width = 50
    
    def _format_watermark_text(self, info: WatermarkInfo, style: WatermarkStyle) -> str:
        """格式化水印文本"""
        text = info.to_string(separator=" | ", compact=True)
        
        if style.prefix:
            text = f"{style.prefix} {text}"
        if style.suffix:
            text = f"{text} {style.suffix}"
        
        return text
    
    def extract(
        self,
        excel_path: Union[str, Path]
    ) -> Optional[WatermarkInfo]:
        """
        从Excel文档中提取水印信息
        
        Args:
            excel_path: 文档路径
            
        Returns:
            提取的水印信息
        """
        wb = load_workbook(excel_path)
        
        for sheet_name in wb.sheetnames:
            ws = wb[sheet_name]
            
            # 从页眉提取
            header_text = ws.oddHeader.center.text
            if header_text and ("用户:" in header_text or "工号:" in header_text):
                return WatermarkInfo.from_string(header_text)
            
            # 从首行提取
            first_row_value = ws.cell(row=1, column=1).value
            if first_row_value and ("用户:" in str(first_row_value) or "工号:" in str(first_row_value)):
                return WatermarkInfo.from_string(str(first_row_value))
            
            # 从批注提取
            if ws['A1'].comment:
                comment_text = ws['A1'].comment.text
                if "用户:" in comment_text or "工号:" in comment_text:
                    # 提取水印部分
                    for line in comment_text.split('\n'):
                        if "用户:" in line or "工号:" in line:
                            return WatermarkInfo.from_string(line.strip())
        
        return None
    
    def batch_embed(
        self,
        input_dir: Union[str, Path],
        output_dir: Union[str, Path],
        info: WatermarkInfo,
        style: Optional[WatermarkStyle] = None,
        recursive: bool = False
    ) -> List[str]:
        """
        批量添加水印
        
        Args:
            input_dir: 输入目录
            output_dir: 输出目录
            info: 水印信息
            style: 水印样式
            recursive: 是否递归处理
            
        Returns:
            处理的文件列表
        """
        input_dir = Path(input_dir)
        output_dir = Path(output_dir)
        
        processed = []
        
        pattern = "**/*.xlsx" if recursive else "*.xlsx"
        
        for input_file in input_dir.glob(pattern):
            if input_file.name.startswith('~$'):  # 跳过临时文件
                continue
            
            relative_path = input_file.relative_to(input_dir)
            output_file = output_dir / relative_path
            
            try:
                self.embed(input_file, output_file, info, style)
                processed.append(str(output_file))
            except Exception as e:
                print(f"处理失败 {input_file}: {e}")
        
        return processed
    
    def get_workbook_info(self, excel_path: Union[str, Path]) -> dict:
        """
        获取Excel工作簿信息
        
        Args:
            excel_path: Excel文件路径
            
        Returns:
            工作簿信息字典
        """
        wb = load_workbook(excel_path, read_only=True)
        
        info = {
            "sheet_names": wb.sheetnames,
            "sheet_count": len(wb.sheetnames),
            "sheets": {}
        }
        
        for sheet_name in wb.sheetnames:
            ws = wb[sheet_name]
            info["sheets"][sheet_name] = {
                "max_row": ws.max_row,
                "max_column": ws.max_column,
            }
        
        wb.close()
        return info



"""
泄密警示明水印SDK安装配置
"""
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="visualwm",
    version="1.0.0",
    author="VisualWM Team",
    author_email="visualwm@example.com",
    description="泄密警示明水印SDK - 支持图片、视频、文本多模态水印嵌入与提取",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/example/visualwm",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Security",
        "Topic :: Multimedia :: Graphics",
        "Topic :: Multimedia :: Video",
    ],
    python_requires=">=3.8",
    install_requires=[
        "Pillow>=9.0.0",
        "opencv-python>=4.5.0",
        "numpy>=1.20.0",
        "python-docx>=0.8.11",
        "PyPDF2>=3.0.0",
        "reportlab>=3.6.0",
        "openpyxl>=3.0.0",
        "lxml>=4.9.0",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
            "black>=22.0.0",
            "flake8>=5.0.0",
        ],
    },
    entry_points={
        "console_scripts": [
            "visualwm=visualwm.cli:main",
        ],
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
VisualWM 示例代码 - 图片水印
"""

from visualwm import ImageWatermark, WatermarkInfo, WatermarkStyle, RiskLevel


def example_basic_image_watermark():
    """基本图片水印示例"""
    print("=" * 60)
    print("示例1: 基本图片水印")
    print("=" * 60)
    
    # 创建水印信息（自动获取IP、MAC、时间戳）
    info = WatermarkInfo(
        username="张三",
        employee_id="EMP001",
        device_id="DEV-2024-001",
        department="研发部",
        auto_fill=True
    )
    
    print(f"水印信息: {info}")
    print(f"校验码: {info.checksum[:8]}")
    
    # 创建水印处理器
    wm = ImageWatermark()
    
    # 使用默认样式添加水印
    # wm.embed("input.jpg", "output.jpg", info)
    
    print("✓ 基本水印配置完成")


def example_custom_style_watermark():
    """自定义样式水印示例"""
    print("\n" + "=" * 60)
    print("示例2: 自定义样式水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="李四",
        employee_id="EMP002",
        auto_fill=True
    )
    
    # 自定义样式
    style = WatermarkStyle(
        font_size=28,
        color=(100, 100, 100),
        opacity=0.4,
        rotation=-45,
        position="tile",
        line_spacing=120,
        column_spacing=250,
    )
    
    print(f"字体大小: {style.font_size}")
    print(f"颜色: {style.hex_color}")
    print(f"透明度: {style.opacity}")
    print(f"旋转角度: {style.rotation}°")
    
    # wm = ImageWatermark()
    # wm.embed("input.jpg", "output_custom.jpg", info, style)
    
    print("✓ 自定义样式配置完成")


def example_high_risk_watermark():
    """高风险文档水印示例"""
    print("\n" + "=" * 60)
    print("示例3: 高风险（机密）文档水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="王五",
        employee_id="EMP003",
        department="安全部",
        auto_fill=True
    )
    
    # 使用高风险预设样式
    style = WatermarkStyle.high_risk_preset()
    
    print(f"前缀: {style.prefix}")
    print(f"颜色: {style.hex_color} (红色)")
    print(f"透明度: {style.opacity}")
    print(f"加粗: {style.bold}")
    print(f"边框: {style.border}")
    
    # wm = ImageWatermark()
    # wm.embed("secret_doc.png", "output_secret.png", info, style)
    
    print("✓ 高风险水印配置完成")


def example_corner_watermark():
    """角落水印示例"""
    print("\n" + "=" * 60)
    print("示例4: 角落水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="赵六",
        employee_id="EMP004",
        auto_fill=True
    )
    
    # 右下角水印
    style = WatermarkStyle.corner_preset("bottom_right")
    
    print(f"位置: {style.position}")
    print(f"边距: {style.margin}px")
    
    # wm = ImageWatermark()
    # wm.embed("input.jpg", "output_corner.jpg", info, style)
    
    print("✓ 角落水印配置完成")


def example_multiline_watermark():
    """多行水印示例"""
    print("\n" + "=" * 60)
    print("示例5: 多行水印")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="孙七",
        employee_id="EMP005",
        device_id="LAPTOP-001",
        auto_fill=True,
        custom_data={
            "项目": "机密项目A",
            "版本": "v1.0"
        }
    )
    
    print("多行水印内容:")
    print(info.to_multiline_string())
    
    # wm = ImageWatermark()
    # wm.add_multiline_watermark("input.jpg", "output_multiline.jpg", info)
    
    print("✓ 多行水印配置完成")


def example_memory_processing():
    """内存处理示例"""
    print("\n" + "=" * 60)
    print("示例6: 内存中处理图片")
    print("=" * 60)
    
    info = WatermarkInfo(username="测试用户", auto_fill=True)
    style = WatermarkStyle.default()
    
    wm = ImageWatermark()
    
    # 读取图片字节
    # with open("input.jpg", "rb") as f:
    #     image_bytes = f.read()
    
    # 在内存中处理
    # watermarked_bytes = wm.embed_to_bytes(image_bytes, info, style, format="PNG")
    
    # 直接使用或保存
    # with open("output_memory.png", "wb") as f:
    #     f.write(watermarked_bytes)
    
    print("✓ 内存处理演示完成")


def example_batch_processing():
    """批量处理示例"""
    print("\n" + "=" * 60)
    print("示例7: 批量处理")
    print("=" * 60)
    
    info = WatermarkInfo(
        username="批量处理用户",
        employee_id="BATCH001",
        auto_fill=True
    )
    
    style = WatermarkStyle.medium_risk_preset()
    
    # wm = ImageWatermark()
    # processed = wm.batch_embed(
    #     input_dir="./input_images/",
    #     output_dir="./output_images/",
    #     info=info,
    #     style=style,
    #     recursive=True
    # )
    # print(f"已处理 {len(processed)} 个文件")
    
    print("✓ 批量处理演示完成")


def example_extract_watermark():
    """提取水印示例"""
    print("\n" + "=" * 60)
    print("示例8: 提取水印（需要OCR）")
    print("=" * 60)
    
    # wm = ImageWatermark()
    
    # 启用OCR提取
    # info = wm.extract("watermarked_image.png", use_ocr=True)
    # if info:
    #     print("提取的水印信息:")
    #     print(info.to_string(separator="\n"))
    # else:
    #     print("未能提取水印信息")
    
    print("注意: OCR提取需要安装 pytesseract 和 Tesseract OCR 引擎")
    print("✓ 提取水印演示完成")


if __name__ == "__main__":
    print("VisualWM 图片水印示例")
    print("=" * 60)
    
    example_basic_image_watermark()
    example_custom_style_watermark()
    example_high_risk_watermark()
    example_corner_watermark()
    example_multiline_watermark()
    example_memory_processing()
    example_batch_processing()
    example_extract_watermark()
    
    print("\n" + "=" * 60)
    print("所有示例演示完成!")
    print("=" * 60)



"""
命令行接口
"""

import argparse
import sys
from pathlib import Path

from visualwm import (
    WatermarkInfo,
    WatermarkStyle,
    RiskLevel,
    ImageWatermark,
    VideoWatermark,
    TextWatermark,
)


def main():
    """命令行入口"""
    parser = argparse.ArgumentParser(
        prog="visualwm",
        description="泄密警示明水印SDK - 支持图片、视频、文本多模态水印嵌入与提取"
    )
    
    subparsers = parser.add_subparsers(dest="command", help="可用命令")
    
    # 添加水印命令
    embed_parser = subparsers.add_parser("embed", help="添加水印")
    embed_parser.add_argument("input", help="输入文件路径")
    embed_parser.add_argument("output", help="输出文件路径")
    embed_parser.add_argument("-u", "--username", required=True, help="用户姓名")
    embed_parser.add_argument("-e", "--employee-id", default="", help="工号")
    embed_parser.add_argument("-d", "--device-id", default="", help="设备编号")
    embed_parser.add_argument("-t", "--type", choices=["image", "video", "text"], help="文件类型（自动检测）")
    embed_parser.add_argument("--risk-level", choices=["low", "medium", "high", "critical"], default="low", help="风险等级")
    embed_parser.add_argument("--position", choices=["tile", "center", "top_left", "top_right", "bottom_left", "bottom_right"], default="tile", help="水印位置")
    embed_parser.add_argument("--opacity", type=float, default=0.5, help="透明度")
    embed_parser.add_argument("--font-size", type=int, default=24, help="字体大小")
    
    # 提取水印命令
    extract_parser = subparsers.add_parser("extract", help="提取水印")
    extract_parser.add_argument("input", help="输入文件路径")
    extract_parser.add_argument("-t", "--type", choices=["image", "video", "text"], help="文件类型")
    extract_parser.add_argument("--ocr", action="store_true", help="使用OCR提取")
    
    # 批量处理命令
    batch_parser = subparsers.add_parser("batch", help="批量添加水印")
    batch_parser.add_argument("input_dir", help="输入目录")
    batch_parser.add_argument("output_dir", help="输出目录")
    batch_parser.add_argument("-u", "--username", required=True, help="用户姓名")
    batch_parser.add_argument("-e", "--employee-id", default="", help="工号")
    batch_parser.add_argument("-t", "--type", choices=["image", "video", "text"], required=True, help="文件类型")
    batch_parser.add_argument("-r", "--recursive", action="store_true", help="递归处理子目录")
    
    # 版本命令
    parser.add_argument("-v", "--version", action="version", version="%(prog)s 1.0.0")
    
    args = parser.parse_args()
    
    if args.command == "embed":
        embed_watermark(args)
    elif args.command == "extract":
        extract_watermark(args)
    elif args.command == "batch":
        batch_process(args)
    else:
        parser.print_help()


def detect_file_type(file_path: str) -> str:
    """检测文件类型"""
    suffix = Path(file_path).suffix.lower()
    
    image_formats = {'.png', '.jpg', '.jpeg', '.bmp', '.gif', '.tiff', '.webp'}
    video_formats = {'.mp4', '.avi', '.mov', '.mkv', '.wmv', '.flv', '.webm'}
    text_formats = {'.txt', '.md', '.rst', '.log', '.html', '.htm'}
    
    if suffix in image_formats:
        return "image"
    elif suffix in video_formats:
        return "video"
    elif suffix in text_formats:
        return "text"
    else:
        raise ValueError(f"无法识别的文件类型: {suffix}")


def embed_watermark(args):
    """添加水印"""
    file_type = args.type or detect_file_type(args.input)
    
    # 创建水印信息
    info = WatermarkInfo(
        username=args.username,
        employee_id=args.employee_id,
        device_id=args.device_id,
        auto_fill=True
    )
    
    # 创建水印样式
    risk_level = RiskLevel(args.risk_level)
    
    if risk_level in [RiskLevel.HIGH, RiskLevel.CRITICAL]:
        style = WatermarkStyle.get_preset_by_risk_level(risk_level)
    else:
        style = WatermarkStyle(
            font_size=args.font_size,
            opacity=args.opacity,
            position=args.position,
            risk_level=risk_level
        )
    
    # 添加水印
    if file_type == "image":
        wm = ImageWatermark()
        output = wm.embed(args.input, args.output, info, style)
    elif file_type == "video":
        wm = VideoWatermark()
        output = wm.embed(args.input, args.output, info, style, 
                         progress_callback=lambda c, t: print(f"\r处理进度: {c}/{t}", end=""))
        print()
    elif file_type == "text":
        wm = TextWatermark()
        output = wm.embed_to_file(args.input, args.output, info)
    
    print(f"水印添加成功: {output}")


def extract_watermark(args):
    """提取水印"""
    file_type = args.type or detect_file_type(args.input)
    
    if file_type == "image":
        wm = ImageWatermark()
        info = wm.extract(args.input, use_ocr=args.ocr)
    elif file_type == "video":
        wm = VideoWatermark()
        info = wm.extract(args.input, use_ocr=args.ocr)
    elif file_type == "text":
        wm = TextWatermark()
        info = wm.extract_from_file(args.input)
    
    if info:
        print("提取的水印信息:")
        print(info.to_string(separator="\n"))
    else:
        print("未找到水印信息")


def batch_process(args):
    """批量处理"""
    info = WatermarkInfo(
        username=args.username,
        employee_id=args.employee_id,
        auto_fill=True
    )
    
    if args.type == "image":
        wm = ImageWatermark()
        processed = wm.batch_embed(args.input_dir, args.output_dir, info, recursive=args.recursive)
    elif args.type == "video":
        wm = VideoWatermark()
        processed = wm.batch_embed(args.input_dir, args.output_dir, info, recursive=args.recursive)
    elif args.type == "text":
        wm = TextWatermark()
        processed = wm.batch_embed(args.input_dir, args.output_dir, info, recursive=args.recursive)
    
    print(f"批量处理完成，共处理 {len(processed)} 个文件")


if __name__ == "__main__":
    main()



"""
VisualWM - 泄密警示明水印SDK

支持图片、视频、文本、文档多模态水印嵌入与提取
"""

from visualwm.core.info import WatermarkInfo
from visualwm.core.style import WatermarkStyle, RiskLevel
from visualwm.core.checksum import ChecksumGenerator
from visualwm.image.watermark import ImageWatermark
from visualwm.video.watermark import VideoWatermark
from visualwm.text.watermark import TextWatermark
from visualwm.document.word import WordWatermark
from visualwm.document.pdf import PDFWatermark
from visualwm.document.excel import ExcelWatermark
from visualwm.document.ofd import OFDWatermark

__version__ = "1.1.0"
__author__ = "VisualWM Team"

__all__ = [
    # 核心模块
    "WatermarkInfo",
    "WatermarkStyle",
    "RiskLevel",
    "ChecksumGenerator",
    # 媒体水印
    "ImageWatermark",
    "VideoWatermark",
    "TextWatermark",
    # 文档水印
    "WordWatermark",
    "PDFWatermark",
    "ExcelWatermark",
    "OFDWatermark",
]
